<?php

namespace Sagaverse\Filecollector\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;

class FilecollectionController extends ActionController {

    /**
     *
     * @var Tx_Filecollector_Service_SessionHandler
     */
    protected $sessionHandler;

    /**
     *
     * @param Tx_Filecollector_Service_SessionHandler $sessionHandler
     */
    private function injectSessionHandler(\Sagaverse\Filecollector\Service\SessionHandler $sessionHandler) {
        $this->sessionHandler = $sessionHandler;
    }

    private function removeFromBasket($basket, $FileUID) {
        // check if file is already in basket
        if (array_key_exists($FileUID, $basket)) {
            // unset file in basket
            unset($basket[$FileUID]);
        }

        // return basket
        return $basket;
    }

    private function addToBasket($basket, $FileUID) {
        // get configuration from extension manager
        $allowedFileTypesPre = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['filecollector']);
        $allowedFileTypes = explode(",",$allowedFileTypesPre['allowedFileTypes']);
        $sys_file_metadata_records = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('sys_file_metadata', 'file', $FileUID);
        $file = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecord('sys_file', $FileUID, 'uid,storage,identifier,extension,name');
        
        // check if file is already in basket && file type allowed
        if (!array_key_exists($FileUID, $basket) && in_array($file['extension'], $allowedFileTypes)) {
            // get file data from typo3 db
            $basket[$FileUID] = $file;
            
            // update title if set
            if (!empty($sys_file_metadata_records[0]['title'])) {
                $basket[$FileUID]['title'] = $sys_file_metadata_records[0]['title'];
            }
        }
        
        // return basket
        return $basket;
    }
    
    private function handleBasket($method, $FileUID) {
        // inject SessionHandler
        if(!method_exists($this, 'sessionHandler')) { 
            $this->injectSessionHandler(New \Sagaverse\Filecollector\Service\SessionHandler);
        }

        // set basket var
        $basket = [];

        // read basket from session
        if ($this->sessionHandler->restoreFromSession('FilecollectorBasket')) {
            $basket = $this->sessionHandler->restoreFromSession('FilecollectorBasket');
        }

        // check for method and $FileUID
        if ($method === 'add' && !empty($FileUID)) {
            $basket = $this->addToBasket($basket, $FileUID);
        } else if ($method === 'remove' && !empty($FileUID)) {
            $basket = $this->removeFromBasket($basket, $FileUID);
        }

        // set new basket if FileUID is set
        if(!empty($FileUID)) {
            $this->sessionHandler->writeToSession($basket, 'FilecollectorBasket');
        }

        // return
        return $basket;
    }

    public function showAction() {
        // handle basket
        $basket = $this->handleBasket('show');
        
        // show basket
        $this->view->assign('basket', $basket);
    }

    public function addAction() {
        // get GET parameter
        $args = $this->request->getArguments();
        $FileUID = intval($args['filecollectionfile']['uid']);
        
        // handle basket
        $basket = $this->handleBasket('add', $FileUID);
        
        // show basket
        $this->view->assign('basket', $basket);
    }

    public function addallAction() {
        // get GET parameters
        $args = $this->request->getArguments();
        $FileListUID = intval($args['filecollectionfile']['uid']); // this is a uid in tt_content
        $basket = $this->handleBasket('show'); // initialize basket

        // get ids of files
        $RowFromTTContent = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('tt_content', 'uid', $FileListUID); // this can be null, empty or a comma seperated list of ids
        $RowsFromSysFileReferenceFileList = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('sys_file_reference', 'uid_foreign', $FileListUID);

        // iterate over rows from sys_file_reference based on UID of FileList
        foreach ($RowsFromSysFileReferenceFileList as $File) {
            // check if file is defined in sys_file_reference
            if ($File['tablenames'] === 'tt_content' && $File['fieldname'] === 'media') {
                // handle basket
                $basket = $this->handleBasket('add', $File['uid_local']);
                print_r($basket);
            }
        }

        // test if there are collections at all
        if (!empty($RowFromTTContent[0]['file_collections'])) {
            // explode comma seperated list of collection ids into an array
            $CollectionIDsArray = explode(",", $RowFromTTContent[0]['file_collections']);
            // iterate over each of the collections
            foreach ($CollectionIDsArray as $CollectionID) {
                // get entries in db
                $FileCollection = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('sys_file_collection', 'uid', $CollectionID);
                if (!empty($FileCollection[0]['type'])) {
                    switch ($FileCollection[0]['type']) {
                        case 'static': // type == static; file ids can be retrieved directly from sys_file_reference
                            $RowsFromSysFileReferenceCollection = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('sys_file_reference', 'uid_foreign', $CollectionID);
                            foreach ($RowsFromSysFileReferenceCollection as $RowFromSysFileReferenceCollection) {
                                $FileUID = $RowFromSysFileReferenceCollection['uid_local'];
                                // check if row is valid
                                if ($RowFromSysFileReferenceCollection['tablenames'] === 'sys_file_collection' && $RowFromSysFileReferenceCollection['fieldname'] === 'files' && $RowFromSysFileReferenceCollection['deleted'] != 1) {
                                    // handle basket
                                    $basket = $this->handleBasket('add', $FileUID);
                                }
                            }
                            break;
                        case 'folder': // type === folder (=>get all files matching folder id)
                            // check if folder is set
                            if (!empty($FileCollection[0]['folder']) && !empty($FileCollection[0]['storage'])) {
                                // get all files from storage
                                $FilesInStorageArray = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('sys_file', 'storage', intval($FileCollection[0]['storage']));
                                foreach ($FilesInStorageArray as $FileInStorage) {
                                    $Identifier = $FileInStorage['identifier'];
                                    $Folder = substr($Identifier, 0, strrpos($Identifier, '/') + 1);
                                    if($Folder === $FileCollection[0]['folder']) {
                                        // handle basket
                                        $basket = $this->handleBasket('add', intval($FileInStorage['uid']));
                                    }
                                }
                            }
                            break;
                        case 'category': // type === category (only 1 is possible)
                            // check if category is set
                            if (!empty($FileCollection[0]['category'])) {
                                // build own select query that does not check if there is a uid (which there is not in an mm table)
                                $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'sys_category_record_mm', 'uid_local = "' . $FileCollection[0]['category'] . '"');
                                while ($record = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                                    if ($record['tablenames'] === 'sys_file_metadata') {
                                        $FileMetadataArray = \TYPO3\CMS\Backend\Utility\BackendUtility::getRecordsByField('sys_file_metadata', 'uid', intval($record['uid_foreign']));
                                        // handle basket
                                        $basket = $this->handleBasket('add', intval($FileMetadataArray[0]['file']));
                                    }
                                }
                                $GLOBALS['TYPO3_DB']->sql_free_result($res);
                            }
                            break;
                    }
                }
            }
        }
        // show basket
        $this->view->assign('basket', $basket);
    }

    public function removeAction() {
        // get GET parameter
        $args = $this->request->getArguments();
        $FileUID = intval($args['filecollectionfile']['uid']);
        
        // handle basket
        $basket = $this->handleBasket('remove', $FileUID);
        
        // show basket
        $this->view->assign('basket', $basket);
    }

    public function downloadAction() {
        // handle basket
        $basket = $this->handleBasket('show');

        // Prepare File
        $file = tempnam("/typo3temp/tx_filecollector", "zip");
        $zip = New \ZipArchive;
        $zip->open($file, \ZipArchive::OVERWRITE);

        // create instance to storage repository
        $storageRepository = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Resource\\StorageRepository');

        // iterate basket
        foreach ($basket AS $filedata) {
            $storageObject = $storageRepository->findByUid($filedata["storage"]);
            $fileObject = $storageObject->getFile($filedata["identifier"]);
            $configuration = $storageObject->getConfiguration();

            // access permissions
            if ($storageObject->isPublic()) {
                $zip->addFile($configuration[basePath] . $filedata["identifier"], $filedata["name"]); // content -> zip
            } else if (!empty($GLOBALS['TSFE']->fe_user->user) && $storageObject->checkFileActionPermission("read", $fileObject) === TRUE) {
                $zip->addFile($configuration[basePath] . $filedata["identifier"], $filedata["name"]); // content -> zip
            }
        }

        // Close and send to users
        $zip->close();

        header('Content-Type: application/zip');
        header('Content-Length: ' . filesize($file));
        header('Content-Disposition: attachment; filename="filecollection.zip"');
        readfile($file);
        unlink($file);

        // stop typo3 from showing any content
        exit;
    }

}

?>